/****************************************************************\
*                                                                *
*  UGAM: UnGapped Alignment Manager                              *
*                                                                *
*  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
*  Copyright (C) 2000-2005.  All Rights Reserved.                *
*                                                                *
*  This source code is distributed under the terms of the        *
*  GNU Lesser General Public License. See the file COPYING       *
*  or http://www.fsf.org/copyleft/lesser.html for details        *
*                                                                *
*  If you use this code, please keep this notice intact.         *
*                                                                *
\****************************************************************/

#ifndef INCLUDED_UGAM_H
#define INCLUDED_UGAM_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#include <glib.h>
#include <limits.h> /* FOR CHAR_BIT */

#include "wordhood.h"
#include "translate.h"
#include "fsm.h"
#include "vfsm.h"
#include "slist.h"
#include "hspset.h"
#include "fastadb.h"
#include "argument.h"
#include "comparison.h"
#include "recyclebin.h"

#ifndef ALPHABET_SIZE
#define ALPHABET_SIZE (1<<CHAR_BIT)
#endif /* ALPHABET_SIZE */

/**/

typedef struct {
       gsize  fsm_memory_limit;
       gchar *force_fsm;
    gboolean  use_wordhood_dropoff;
        gint  word_jump;
        /**/
        gint  dna_word_threshold;
        gint  protein_word_threshold;
} UGAM_ArgumentSet;

UGAM_ArgumentSet *UGAM_ArgumentSet_create(Argument *arg);

/**/

typedef void (*UGAM_ReportFunc)(Comparison *comparison,
                                gpointer user_data);

typedef struct {
        Sequence *query;
      Comparison *curr_comparison;
} UGAM_Matcher_QueryInfo;

typedef struct UGAM_Matcher_QPos {
    struct UGAM_Matcher_QPos *next;
                        gint  query_pos;
      UGAM_Matcher_QueryInfo *query_info;
} UGAM_Matcher_QPos;

typedef struct UGAM_Matcher_NPos {
        struct UGAM_Matcher_NPos *next;
    struct UGAM_Matcher_WordInfo *neighbour;
} UGAM_Matcher_NPos;

typedef struct UGAM_Matcher_WordInfo {
    UGAM_Matcher_QPos *query_pos_list;
    UGAM_Matcher_NPos *wordhood_list;  /* Absent w/o wordhood  ? */
                 gint  match_count;    /* Absent w/o sat_thold ? */
                 gint  match_mailbox;  /* Absent w/o sat_thold ? */
} UGAM_Matcher_WordInfo;

typedef struct {
                 gboolean   is_prepared;
                GPtrArray  *query_info_list;
                      FSM  *fsm; /* Either fsm or vfsm will be NULL */
                     VFSM  *vfsm;
    UGAM_Matcher_WordInfo **vfsm_leaf;
               RecycleBin  *recycle_wordinfo;
               RecycleBin  *recycle_qpos;
               RecycleBin  *recycle_npos;
                     gint   total_query_length;
                     gint   query_expect;
                     gint   comparison_count;
} UGAM_Matcher;

/**/

typedef struct {
    gchar *alphabet;
     gint  alphabet_size;
    guint  wordlen;
    gchar  index[ALPHABET_SIZE];
} UGAM_StrandParam;

typedef struct {
    UGAM_StrandParam *query_param;
    UGAM_StrandParam *target_param;
                gint  word_threshold;
} UGAM_Param;

typedef struct UGAM {
      UGAM_ArgumentSet *uas;
      Comparison_Param *comparison_param;
            UGAM_Param *param;
       UGAM_ReportFunc  report_func;
              WordHood *word_hood;
              gpointer  user_data; /* For UGAM_ReportFunc */
                  gint  verbosity;
                  gint  saturate_threshold;
              gboolean  scan_query;
                  /**/
          UGAM_Matcher *curr_matcher;
                  gint  tpos_modifier;
                  gint  seed_len;
} UGAM;

UGAM *UGAM_create(Comparison_Param *comparison_param,
                  UGAM_ReportFunc report_func,
                  gpointer user_data,
                  gint dna_wordlen, gint protein_wordlen,
                  gint saturate_threshold, gboolean scan_query,
                  gint verbosity);
void  UGAM_destroy(UGAM *ugam);

    void UGAM_query_prepare(UGAM *ugam);
gboolean UGAM_query_present(UGAM *ugam, FastaDB_Seq *query);
    void UGAM_target_prepare(UGAM *ugam);
    void UGAM_target_present(UGAM *ugam, FastaDB_Seq *target);
    void UGAM_clear(UGAM *ugam);
/* FIXME: change to take Sequence not FastaDB_Seq */


/* UGAM_query_prepare() must be called before presentation of queries
 * UGAM_query_present() will return FALSE when the memory limit
 *                      is reached
 * UGAM_target_prepare() must be called before presentation of targets
 * UGAM_target_present() triggers comparison against
 *                       all the current queries
 * UGAM_clear() will clear the current queries
 */

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* INCLUDED_UGAM_H */

