/****************************************************************\
*                                                                *
*  UGAM: UnGapped Alignment Manager                              *
*                                                                *
*  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
*  Copyright (C) 2000-2005.  All Rights Reserved.                *
*                                                                *
*  This source code is distributed under the terms of the        *
*  GNU Lesser General Public License. See the file COPYING       *
*  or http://www.fsf.org/copyleft/lesser.html for details        *
*                                                                *
*  If you use this code, please keep this notice intact.         *
*                                                                *
\****************************************************************/

#include "ugam.h"
#include "submat.h"
#include "translate.h"
#include "alphabet.h"

static void UGAM_test_report_func(Comparison *comparison,
                                  gpointer user_data){
    g_message("Have hsp_set");
    HSPset_print(comparison->hspset);
    return;
    }

int Argument_main(Argument *arg){
    register UGAM *ugam;
    register gchar *query_path, *target_path;
    register Alphabet_Type query_type, target_type;
    register FastaDB_Seq *query_fdbs, *target_fdbs;
    register Submat *dna_submat, *protein_submat;
    register Translate *translate;
    register Alphabet *query_alphabet, *target_alphabet;
    register Comparison_Param *comparison_param;
    if(arg->argc == 5){
        query_path = arg->argv[1];
        target_path = arg->argv[2];
        query_type = (arg->argv[3][0] == 'd')?Alphabet_Type_DNA
                                             :Alphabet_Type_PROTEIN;
        target_type = (arg->argv[4][0] == 'd')?Alphabet_Type_DNA
                                              :Alphabet_Type_PROTEIN;
        g_message("Creating UGAM with [%s][%s]",
                  query_path, target_path);
        query_alphabet = Alphabet_create(query_type, FALSE);
        target_alphabet = Alphabet_create(target_type, FALSE);
        query_fdbs = FastaDB_get_single(query_path, query_alphabet);
        target_fdbs = FastaDB_get_single(target_path, target_alphabet);
        Alphabet_destroy(query_alphabet);
        Alphabet_destroy(target_alphabet);
        dna_submat = Submat_create("nucleic");
        protein_submat = Submat_create("blosum62");
        translate = Translate_create(FALSE);
        comparison_param = Comparison_Param_create(
                query_type, target_type, dna_submat, protein_submat,
                translate, FALSE, TRUE, 1);
        ugam = UGAM_create(comparison_param, UGAM_test_report_func,
                          NULL, 11, 5, 0, 1, 1);
        Comparison_Param_destroy(comparison_param);
        Submat_destroy(dna_submat);
        Submat_destroy(protein_submat);
        Translate_destroy(translate);
        UGAM_query_prepare(ugam);
        UGAM_query_present(ugam, query_fdbs);
        UGAM_target_prepare(ugam);
        UGAM_target_present(ugam, target_fdbs);
        UGAM_clear(ugam);
        UGAM_destroy(ugam);
        FastaDB_Seq_destroy(query_fdbs);
        FastaDB_Seq_destroy(target_fdbs);
    } else {
        g_message("Usage: <query_path> <target_path>\n"
         "                <query_type> <target_type> (type==[d|p]");
        }
    return 0;
    }

