/****************************************************************\
*                                                                *
*  UTR alignment model                                           *
*                                                                *
*  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
*  Copyright (C) 2000-2005.  All Rights Reserved.                *
*                                                                *
*  This source code is distributed under the terms of the        *
*  GNU Lesser General Public License. See the file COPYING       *
*  or http://www.fsf.org/copyleft/lesser.html for details        *
*                                                                *
*  If you use this code, please keep this notice intact.         *
*                                                                *
\****************************************************************/

#include "utr.h"

UTR_Data *UTR_Data_create(Sequence *query, Sequence *target){
    register UTR_Data *utrd = g_new(UTR_Data, 1);
    g_assert(query->alphabet->type == Alphabet_Type_DNA);
    g_assert(target->alphabet->type == Alphabet_Type_DNA);
    Coding2Genome_Data_init(&utrd->c2gd, query, target);
    return utrd;
    }

void UTR_Data_destroy(UTR_Data *utrd){
    Coding2Genome_Data_clear(&utrd->c2gd);
    g_free(utrd);
    return;
    }

/**/

/* Create a special single-orientations e2g model */
static C4_Model *UTR_est2genome_create(void){
    register C4_Model *model = Affine_create(
                Affine_Model_Type_LOCAL,
                Alphabet_Type_DNA,
                Alphabet_Type_DNA, FALSE);
    /* FIXME: add introns to e2g_model */
    register C4_Model *intron_model = Intron_create("forward",
                                                    FALSE, TRUE);
    register C4_State *match_state
           = C4_Model_find_state(model, "match");
    C4_Model_open(model);
    C4_Model_insert(model, intron_model, match_state, match_state);
    C4_Model_close(model);
    C4_Model_destroy(intron_model);
    return model;
    }

C4_Model *UTR_create(void){
    register C4_Model *model = Coding2Genome_create();
    register C4_Model *e2g_model = UTR_est2genome_create();
    register C4_State *match_state_a, *match_state_b;
    g_assert(model);
    C4_Model_rename(model, "utr");
    C4_Model_open(model);
    /**/
    C4_Model_insert(model, e2g_model, NULL, NULL);
    g_assert(model->start_state->state->output_transition_list->len
             == 2);
    match_state_a = ((C4_Transition*)
        model->start_state->state->output_transition_list->pdata[0])
        ->output;
    match_state_b = ((C4_Transition*)
        model->start_state->state->output_transition_list->pdata[1])
        ->output;
    g_assert(match_state_a);
    g_assert(match_state_b);
    /* Add c2g<->e2g transitions */
    /* FIXME: transitions are wrong (but cannot be silent) */
    C4_Model_add_transition(model, "utr transition a",
            match_state_a, match_state_b, 1, 1, NULL, C4_Label_NONE);
    C4_Model_add_transition(model, "utr transition b",
            match_state_b, match_state_a, 3, 3, NULL, C4_Label_NONE);
    /* FIXME: add coding constraints on match transitions */
    C4_Model_destroy(e2g_model);
    /**/
    C4_Model_close(model);
    C4_Model_dump_graphviz(model);
    return model;
    }

/**/

