/****************************************************************\
*                                                                *
*  UTR alignment model                                           *
*                                                                *
*  Guy St.C. Slater..   mailto:guy@ebi.ac.uk                     *
*  Copyright (C) 2000-2005.  All Rights Reserved.                *
*                                                                *
*  This source code is distributed under the terms of the        *
*  GNU Lesser General Public License. See the file COPYING       *
*  or http://www.fsf.org/copyleft/lesser.html for details        *
*                                                                *
*  If you use this code, please keep this notice intact.         *
*                                                                *
\****************************************************************/

#include "utr.h"
#include "alignment.h"
#include "optimal.h"
#include "submat.h"

static void test_utr(Sequence *query, Sequence *target){
    register C4_Score score;
    register C4_Model *model = UTR_create();
    register UTR_Data *utrd = UTR_Data_create(query, target);
    register Alignment *alignment;
    register Optimal *optimal = Optimal_create(model, NULL,
                                               Optimal_Type_SCORE
                                              |Optimal_Type_PATH,
                                              FALSE);
    register Region *region = Region_create(0, 0,
                                            query->len, target->len);
    score = Optimal_find_score(optimal, region, utrd, NULL);
    g_message("Score is [%d]", score);
    /* g_assert(score == 151); */
    alignment = Optimal_find_path(optimal, region, utrd,
                                  C4_IMPOSSIBLY_LOW_SCORE, NULL);
    g_message("Alignment score is [%d]", alignment->score);
    Alignment_display(alignment, query, target,
                     UTR_Data_get_submat(utrd),
                     UTR_Data_get_translate(utrd));
    g_assert(score == alignment->score);
    UTR_Data_destroy(utrd);
    C4_Model_destroy(model);
    Region_destroy(region);
    Optimal_destroy(optimal);
    return;
    }

int Argument_main(Argument *arg){
    register Alphabet *alphabet = Alphabet_create(Alphabet_Type_DNA,
                                                  FALSE);
    register Sequence
        *qy = Sequence_create("qyr", NULL,
"AGCCCAGCCAAGCACTGTCAGGAATCCTGT"
"GAAGCAGCTCCAGCTATGTGTGAAGAA"
"GAGGACAGCACTGCCTTGGTGTGTGACAATG"
/* "GTNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNAG" */
"GCTCTGGGCTCTGTAAGGCCGGCTTTGCT", 0,
                     Sequence_Strand_UNKNOWN, alphabet),
        *tg = Sequence_create("tgr", NULL,
"AGCCCAGCCAAGCACTGTCAGGAATCCTG"
"GTNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNAG"
"T"
"GAAGCAGCTCCAGCTATGTGTGAAGAA"
"GAGGACAGCACTGCCTTGGTGTGTGACAATGGC"
"TCTGGGCTCTGTAAGGCCGGCTTTGCT", 0,
                     Sequence_Strand_UNKNOWN, alphabet);
    Ungapped_ArgumentSet_create(arg);
    Affine_ArgumentSet_create(arg);
    Viterbi_ArgumentSet_create(arg);
    Frameshift_ArgumentSet_create(arg);
    Intron_ArgumentSet_create(arg);
    Argument_process(arg, "utr.test", NULL, NULL);
    g_message("Testing utr");
    g_message("seq lengths [%d,%d]", qy->len, tg->len);
    test_utr(qy, tg);
    Sequence_destroy(qy);
    Sequence_destroy(tg);
    Alphabet_destroy(alphabet);
    return 0;
    }


